from functools import partial

import pytest
from flask.testing import FlaskClient

from nidhoggr.bl import BLConfig
from nidhoggr.views import session, legacy
from ..conftest import accessor, accessor_legacy, EndpointCallable


@pytest.fixture
def join(client: FlaskClient) -> EndpointCallable:
    return partial(accessor(session.join), client)


@pytest.fixture
def has_joined(client: FlaskClient, config: BLConfig) -> EndpointCallable:
    if config.legacy_compat:
        return partial(accessor_legacy(legacy.hasJoined), client)
    else:
        return partial(accessor(session.has_joined), client)


@pytest.fixture
def profile(client: FlaskClient, config: BLConfig) -> EndpointCallable:
    if config.legacy_compat:
        return partial(accessor_legacy(legacy.profile), client)
    else:
        return partial(accessor(session.profile), client)



from typing import Union

from nidhoggr.bl import BaseUserRepo, BLConfig, BaseTextureRepo
from nidhoggr.errors.common import YggdrasilError
from nidhoggr.models.session import HasJoinedRequestLegacy, ProfileRequestLegacy, JoinedResponse
from nidhoggr.utils.decorator import typed
from nidhoggr.views import session


# noinspection PyPep8Naming
@typed
def hasJoined(req: HasJoinedRequestLegacy, users: BaseUserRepo) -> Union[JoinedResponse, YggdrasilError]:
    return session.has_joined.__wrapped__(req, users)


@typed
def profile(
    req: ProfileRequestLegacy,
    users: BaseUserRepo,
    config: BLConfig,
    textures: BaseTextureRepo
) -> Union[JoinedResponse, YggdrasilError]:
    return session.profile.__wrapped__(req, users, config, textures)


__all__ = {"hasJoined", "profile"}



from base64 import b64encode
from enum import Enum
from typing import Optional, Dict

from pydantic import BaseModel

from nidhoggr.models.session import UserProperty
from nidhoggr.utils.transformer import JSONResponseTransformer

TextureMeta = Optional[Dict[str, str]]


class TextureType(Enum):
    SKIN = "SKIN"
    CAPE = "CAPE"
    ELYTRA = "ELYTRA"


class TextureItem(BaseModel):
    url: str
    metadata: TextureMeta = None

    class Config:
        allow_mutation = False


class TextureResponse(BaseModel, JSONResponseTransformer):
    timestamp: int
    profileId: str
    profileName: str
    textures: Dict[TextureType, TextureItem]

    class Config:
        allow_mutation = False
        use_enum_values = True

    def pack(self) -> UserProperty:
        return UserProperty(
            name="textures",
            value=b64encode(self.json().encode('ascii')).decode('ascii')
        )


class TextureToken(BaseModel):
    name: str
    ip: str
    server_id: str
    value: str

    class Config:
        allow_mutation = False



from flask import Flask
from werkzeug.exceptions import NotFound, MethodNotAllowed, InternalServerError

from nidhoggr.bl import BaseUserRepo, BLConfig, BaseTextureRepo
from nidhoggr.views import auth, session, core, legacy


def configure_error_handlers(app: Flask):
    app.register_error_handler(NotFound, core.not_found)
    app.register_error_handler(MethodNotAllowed, core.method_not_allowed)
    app.register_error_handler(InternalServerError, core.internal_server_error)


def configure_views(app: Flask, legacy_compat: bool):
    for func in auth.__all__:
        app.add_url_rule(f"/{func}", func, getattr(auth, func), methods=["POST"])

    app.add_url_rule("/join", "join", session.join, methods=["POST"])
    if legacy_compat:
        app.add_url_rule("/hasJoined", "hasJoined", legacy.hasJoined, methods=["GET"])
        app.add_url_rule("/profile", "profile", legacy.profile, methods=["GET"])
    else:
        app.add_url_rule("/has_joined", "has_joined", session.has_joined, methods=["POST"])
        app.add_url_rule("/profile", "profile", session.profile, methods=["POST"])


def create_app(users: BaseUserRepo, config: BLConfig, textures: BaseTextureRepo) -> Flask:
    app = Flask(__package__)
    app.bl = {
        BaseUserRepo: users,
        BLConfig: config,
        BaseTextureRepo: textures
    }
    configure_error_handlers(app)
    configure_views(app, config.legacy_compat)
    return app
